@@ -32,6 +32,7 @@ typedef struct {
 	int hilines;	/* nr of lines max article length */
 	int lowlines;	/* nr of lines min article length */
 	int maxgrps;	/* max nr of grps (to prevent spams) */
+	int maxxref;    /* max nr of Xrefs (another spamer) */
 	unsigned long bodybig;	/* max size of article body  */
 	unsigned long bodysmall; /* minimum size of article body */
 	char quote;	/* character to use as quote (for case compare) */
@@ -70,6 +71,7 @@ typedef struct killstruct {
 	int tie_delete;
 	int totgrps;
 	int ignore_postfix;
+	int xover_log_long;
 	PGroup grps;	/* dynamicly allocated array */
 	int ( *killfunc)(PMaster, struct killstruct *, char *, int); /*function to call to check header */
 	char *pbody;
@@ -84,7 +86,7 @@ typedef struct killstruct {
 } KillStruct, *PKillStruct;
 
 /* function prototypes for killfile.c  and xover.c*/
-int get_one_article_kill(PMaster, int);
+int get_one_article_kill(PMaster, int, long);
 PKillStruct parse_killfile(const char *, int, int, int);
 void free_killfile(PKillStruct);
 int chk_msg_kill(PMaster, PKillStruct, char *, int);
@@ -103,4 +105,9 @@ void killperl_done(PKillStruct);
 
 enum { KILL_LOG_NONE, KILL_LOG_SHORT, KILL_LOG_LONG };  /* what level of logging do we use */
 enum { DELKEEP_KEEP, DELKEEP_DELETE }; /* is this a keep or delete group */
+
+#define COMMA ','			/* group separator in newsgroup line in article */
+#define SPACE ' '                       /* another group separator in newsgroup/xref line in article */
+#define COLON ':'                       /* group/article nr separator in Xref line */
+
 #endif /* _SUCK_KILLFILE_H */
