@@ -56,6 +56,8 @@
 #include <perl.h>
 #ifndef ERRSV
 # define ERRSV (GvSV(errgv))  /* needed for perl 5.004 and earlier */
+#endif
+#ifndef PL_na
 # define PL_na (na)
 #endif
 #endif
@@ -149,6 +151,7 @@ int killprg_forkit(PKillStruct mkill, char *args, int debug) {
 				dup2(newstdin[0], 0);	/* copy new read to stdin */
 				dup2(newstdout[1], 1);	/* copy new write to stdout */
 				execvp(prg[0], prg);
+				exit(-1); /* so if exec fails, we don't get two sucks running */
 			}
 			else if( mkill->child.Pid == -1) {
 			  	/* whoops */
