@@ -44,6 +44,7 @@ typedef struct {
 	FILE *msgs;
 	int nrgroups;
 	int debug;
+	int abort_exist; /* do we abort if article nr already exists */
 	PGroup groups;
 	char active_file[PATH_MAX+1];
 	const char *config_file;
@@ -97,6 +98,7 @@ int main(int argc, char *argv[]) {
 	master.phrases = NULL;
 	master.basedir[0] = '\0';
 	master.msgdir = NULL;
+	master.abort_exist = TRUE;
 
 	scan_args(&master, argc, argv);
 	load_phrases(&master);	/* this has to be here so rest prints okay */
@@ -111,7 +113,12 @@ int main(int argc, char *argv[]) {
 		do_debug("master.msgdir = %s\n", (master.msgdir == NULL) ? "NULL" : master.msgdir);
 		do_debug("master.phrases = %s\n", (master.phrases == NULL) ? "NULL" :master.phrases);
 	}
-	if(master.basedir[0] == '\0' || master.msgdir[0] == '\0') {
+	if(master.basedir == NULL || master.basedir[0] == '\0') {
+		error_log(ERRLOG_REPORT, lmove_phrases[5], NULL);
+		retval = RETVAL_ERROR;
+	}
+	if(master.msgdir == NULL || master.msgdir[0] == '\0') {
+		error_log(ERRLOG_REPORT, lmove_phrases[7], NULL);
 		retval = RETVAL_ERROR;
 	}
 	
@@ -181,16 +188,15 @@ int move_msgs(PMaster master) {
 	char *ptr;
 	PGroup group;
 	char o_fname[PATH_MAX+1], n_fname[PATH_MAX+1];
-
-
-	int retval = RETVAL_OK;
+	struct stat sbuf;
+	int quit = FALSE, retval = RETVAL_OK;
 
 	if((dptr = opendir(master->msgdir)) == NULL) {
 		MyPerror(master->msgdir);
 		retval = RETVAL_ERROR;
 	}
 	else {
-		while((entry = readdir(dptr)) != NULL) {
+		while(((entry = readdir(dptr)) != NULL) && quit == FALSE) {
 			sprintf(o_fname, "%s/%s", master->msgdir, entry->d_name);	/* build full path */
 			if(stat(o_fname, &statbuf) == 0 && S_ISREG(statbuf.st_mode)) {
 				if((ptr = find_groups(master, o_fname)) == NULL) {
@@ -200,17 +206,31 @@ int move_msgs(PMaster master) {
 					error_log(ERRLOG_REPORT, lmove_phrases[12], entry->d_name, NULL);
 				}
 				else {
-					/* try the move */
-					/* make destination filename */
 					sprintf(n_fname, "%s/%d", make_dirname(master->basedir, group->group), group->highnr);
-					if(rename(o_fname, n_fname) != 0) {
-						error_log(ERRLOG_REPORT, lmove_phrases[13], o_fname, n_fname, NULL);
-						MyPerror("");
+					if(stat(n_fname, &sbuf) == 0) {
+						/* file exists */
+						if(master->abort_exist == TRUE) {
+							error_log(ERRLOG_REPORT, lmove_phrases[21], group->group, str_int(group->highnr), NULL);
+							quit = TRUE;
+						}
+						else {
+							while(stat(n_fname, &sbuf) == 0) {
+								/* up it and try again */
+								group->highnr++;
+								sprintf(n_fname, "%s/%d", make_dirname(master->basedir, group->group), group->highnr);
+							}
+						}
 					}
-					else {
-						group->highnr++;
+					if(quit != TRUE) {
+						if(rename(o_fname, n_fname) != 0) {
+							error_log(ERRLOG_REPORT, lmove_phrases[13], o_fname, n_fname, NULL);
+							MyPerror("");
+						}
+						else {
+							group->highnr++;
+						}
 					}
-				}
+				}				
 			}
 		}
 		closedir(dptr);
@@ -454,29 +474,7 @@ void scan_args(PMaster master, int argc, char *argv[]) {
 		}
 		else {
 			switch(argv[loop][1]) {
-			  case 'e':	/* use default error log path */
-				error_log(ERRLOG_SET_FILE, ERROR_LOG,NULL);
-				break;
-			  case 'E':	/* error log path */
-				if(loop+1 == argc) { 
-					error_log(ERRLOG_REPORT, "%s\n",lmove_phrases[1],NULL);
-				}
-				else {
-					error_log(ERRLOG_SET_FILE, argv[++loop], NULL);
-				}
-				break;
-			  case 'l': 	/* language  phrase file */
-				if(loop+1 == argc) {
-					error_log(ERRLOG_REPORT, lmove_phrases[4],NULL);
-				}
-				else {
-					master->phrases = argv[++loop];
-				}
-				break;
-			  case 'D':	/* debug */
-				master->debug = TRUE;
-				break;
-			  case 'a':	/* active file */
+			case 'a':	/* active file */
 				if(loop+1 == argc) {
 					error_log(ERRLOG_REPORT, lmove_phrases[6], NULL);
 				}
@@ -484,7 +482,10 @@ void scan_args(PMaster master, int argc, char *argv[]) {
 					strcpy(master->active_file,argv[++loop]);
 				}
 				break;
-			  case 'c': /* config file */
+			case 'A': /* don't abort if article nr already exists on disk */
+				master->abort_exist = FALSE;
+				break;
+			case 'c': /* config file */
 				if(loop+1 == argc) {
 					error_log(ERRLOG_REPORT, lmove_phrases[19], NULL);
 				}
@@ -492,7 +493,7 @@ void scan_args(PMaster master, int argc, char *argv[]) {
 					master->config_file =argv[++loop];
 				}
 				break;
-			  case 'd': 	/* article directory */
+			case 'd': 	/* article directory */
 				if(loop+1 == argc) {
 					error_log(ERRLOG_REPORT, lmove_phrases[7], NULL);
 				}
@@ -500,7 +501,29 @@ void scan_args(PMaster master, int argc, char *argv[]) {
 					master->msgdir = argv[++loop];
 				}
 				break;
-			  default:
+			case 'D':	/* debug */
+				master->debug = TRUE;
+				break;
+			case 'e':	/* use default error log path */
+				error_log(ERRLOG_SET_FILE, ERROR_LOG,NULL);
+				break;
+			case 'E':	/* error log path */
+				if(loop+1 == argc) { 
+					error_log(ERRLOG_REPORT, "%s\n",lmove_phrases[1],NULL);
+				}
+				else {
+					error_log(ERRLOG_SET_FILE, argv[++loop], NULL);
+				}
+				break;
+			case 'l': 	/* language  phrase file */
+				if(loop+1 == argc) {
+					error_log(ERRLOG_REPORT, lmove_phrases[4],NULL);
+				}
+				else {
+					master->phrases = argv[++loop];
+				}
+				break;
+			default:
 				error_log(ERRLOG_REPORT, lmove_phrases[0], argv[loop],NULL);
 				break;
 			
