@@ -21,6 +21,8 @@
 #include <perl.h>
 #ifndef ERRSV
 # define ERRSV (GvSV(errgv))  /* needed for perl 5.004 and earlier */
+#endif
+#ifndef PL_na
 # define PL_na (na)
 #endif
 #endif
@@ -495,7 +497,7 @@ int do_filter(int argc, char *argv[], int debug) {
 	  case 0:	/* in child, do execl */
 		if(execvp(argv[0], argv) == -1) {
 			MyPerror(rpost_phrases[14]);
-			retval = RETVAL_ERROR;
+			exit(-1); /* so we don't get two running */
 		}
 		break;
 	  case -1:	/* should never get here */
