@@ -44,6 +44,7 @@
 #include "active.h"
 #include "batch.h"
 #include "xover.h"
+#include "db.h"
 
 #ifdef MYSIGNAL 
 #include <signal.h>
@@ -55,23 +56,21 @@
 
 /* function prototypes */
 int get_articles(PMaster);
-int get_one_article(PMaster, int);
-int do_connect(PMaster, int);
+int get_one_article(PMaster, int, long);
 int do_supplemental(PMaster);
 int restart_yn(PMaster);
 int scan_args(PMaster, int, char *[]);
-void do_cleanup(void);
+void do_cleanup(PMaster);
 int do_authenticate(PMaster);
 void load_phrases(PMaster);
 void free_phrases(void);
 int parse_args(PMaster, int, char *[]);
 int get_group_number(PMaster, char *);
-
+int do_supplemental(PMaster);
 #ifdef MYSIGNAL
 RETSIGTYPE sighandler(int);
 void pause_signal(int, PMaster);
 enum {PAUSE_SETUP, PAUSE_DO};
-enum {CONNECT_FIRST, CONNECT_AGAIN};
 
 /*------------------------------------------*/
 int GotSignal = FALSE;		
@@ -107,7 +106,8 @@ enum {
 	ARG_KLOG_LONG, ARG_MODEREADER, ARG_PORTNR, ARG_PASSWD, ARG_RESCAN, ARG_STATLOG, \
 	ARG_USERID, ARG_VERSION, ARG_WAIT, ARG_LOCALHOST, ARG_TIMEOUT, ARG_NRMODE, ARG_AUTOAUTH, \
 	ARG_NODEDUPE, ARG_NO_CHK_MSGID, ARG_READACTIVE, ARG_PREBATCH, ARG_SKIP_ON_RESTART, \
-	ARG_KLOG_NAME, ARG_USEGUI, ARG_XOVER, ARG_CONN_DEDUPE
+	ARG_KLOG_NAME, ARG_USEGUI, ARG_XOVER, ARG_CONN_DEDUPE, ARG_POST_FILTER, ARG_CONN_ACTIVE,
+	ARG_HIST_FILE,
 }; 
 
 typedef struct Arglist{
@@ -143,7 +143,8 @@ const Args arglist[] = {
 	{"s",  "def_status_log", 0, ARG_DEF_STATLOG, -1},
 	{"u",  "auto_authorization", 0, ARG_AUTOAUTH, -1},
 	{"w",  "wait_signal", 2, ARG_WAIT_SIG, 46},
-	{"x",  "no_chk_msgid", 0, ARG_NO_CHK_MSGID, -1},	
+	{"x",  "no_chk_msgid", 0, ARG_NO_CHK_MSGID, -1},
+	{"y",  "post_filter", 1, ARG_POST_FILTER, 62},	
 	{"z",  "no_dedupe", 0, ARG_NODEDUPE, -1},	
 	{"A",  "active", 0, ARG_ACTIVE, -1},
 	{"AL", "read_active", 1, ARG_READACTIVE, 56},
@@ -151,8 +152,10 @@ const Args arglist[] = {
 	{"C",  "reconnect", 1, ARG_RECONNECT, 49},
 	{"D",  "debug", 0, ARG_DEBUG, -1},
 	{"E",  "error_log", 1, ARG_ERRLOG, 41},
+	{"F",  "reconnect_active", 0, ARG_CONN_ACTIVE, -1},
 	{"G",  "use_gui", 0, ARG_USEGUI, -1},
 	{"H",  "no_history", 0, ARG_HISTORY, -1},
+	{"HF", "history_file", 1, ARG_HIST_FILE, 64},
 	{"K",  "killfile", 0, ARG_KILLFILE, -1},
 	{"L",  "kill_log_none", 0, ARG_KLOG_NONE, -1},
 	{"LF", "kill_log_name", 1, ARG_KLOG_NAME, 61},
@@ -192,7 +195,7 @@ int main(int argc, char *argv[]) {
 
 	/* initialize master structure */
 	master.head = master.curr = NULL;
-	master.nritems =  master.itemon = master.nrgot = 0;
+	master.nritems = master.nrgot = 0;
 	master.MultiFile = FALSE;
 	master.msgs = stdout;
 	master.sockfd = -1;
@@ -239,6 +242,10 @@ int main(int argc, char *argv[]) {
 	master.use_gui = FALSE;
 	master.do_xover = TRUE;
 	master.conn_dedupe = FALSE;
+	master.post_filter= NULL;
+	master.conn_active = FALSE;
+	master.history_file = HISTORY_FILE;
+	master.db = -1;
 	
 	/* have to do this next so if set on cmd line, overrides this */
 
@@ -331,6 +338,9 @@ int main(int argc, char *argv[]) {
 		do_debug("master.use_gui = %s\n", true_str(master.use_gui));
 		do_debug("master.do_xover = %s\n", true_str(master.do_xover));
 		do_debug("master.conn_dedupe = %s\n", true_str(master.conn_dedupe));
+		do_debug("master.post_filter = %s\n", null_str(master.post_filter));
+		do_debug("master.conn_active = %s\n", true_str(master.conn_active));
+		do_debug("master.history_file = %s\n", null_str(master.history_file));
 		
 #ifdef TIMEOUT
 		do_debug("TimeOut = %d\n", TimeOut);
@@ -419,7 +429,7 @@ int main(int argc, char *argv[]) {
 		if(do_connect(&master, CONNECT_FIRST) != RETVAL_OK) {
 			retval = RETVAL_ERROR;
 		}
-		else {
+		else {	 
 			/* if we have XOVER killfiles, we need to get the format of em before any processing */
 			if(master.xoverp != NULL) {
 				get_xoverview(&master);
@@ -441,19 +451,39 @@ int main(int argc, char *argv[]) {
 				else if(get_message_index(&master) < RETVAL_OK) {
 					retval = RETVAL_ERROR;
 				}
+				if(retval == RETVAL_OK) {
+					retval = do_supplemental(&master);
+				}
+				if(retval == RETVAL_OK && master.head != NULL && master.nritems > 0) {
+					/* if we don't have any messages, we don't need to do this */
+					if(master.no_dedupe == FALSE) {
+						dedupe_list(&master);
+					}
+#ifdef CHECK_HISTORY
+					if(master.do_chkhistory == TRUE) {
+						chkhistory(&master);
+					}
+#endif
+					print_phrases(master.msgs,suck_phrases[20], str_int(master.nritems), NULL);
+				}
 			}
 			if(retval == RETVAL_OK) {
 				if(master.nritems == 0) {
  					print_phrases(master.msgs,suck_phrases[3], NULL);
 					retval = RETVAL_NOARTICLES;
 				}
+				else {
+					/* write out restart db */
+					retval = db_write(&master);
 				/* reconnect after dedupe, in case of time out due to long dedupe time*/
-				if(master.conn_dedupe == TRUE) {
-					retval = do_connect(&master, CONNECT_AGAIN);
-				}
-				if(retval == RETVAL_OK) {
-					retval = get_articles(&master);
+					if(retval == RETVAL_OK && master.conn_dedupe == TRUE) {
+						retval = do_connect(&master, CONNECT_AGAIN);
+					}
+					if(retval == RETVAL_OK) {
+						retval = get_articles(&master);
+					}
 				}
+				
 			}
 			/* send quit, and get reply */
 			sputline(master.sockfd,"quit\r\n");
@@ -466,8 +496,7 @@ int main(int argc, char *argv[]) {
 					if(master.debug == TRUE) {
 						do_debug("Quitting GOT: %s", inbuf);
 					}
-					number(inbuf, &nr);
-					
+					number(inbuf, &nr);					
 				}
 			
 			}while(nr != 205 && resp > 0);	
@@ -482,25 +511,30 @@ int main(int argc, char *argv[]) {
 		if((retval == RETVAL_OK || master.always_batch == TRUE) && master.nrgot > 0) {
 			switch(master.batch) {
 			  case BATCH_INNXMIT:
-				do_innbatch(&master);
-				break;
+				  do_post_filter(&master);
+				  do_innbatch(&master);
+				  break;
 			  case BATCH_RNEWS:
-				do_rnewsbatch(&master);
-				break;
+				  do_post_filter(&master);
+				  do_rnewsbatch(&master);
+				  break;
 			  case BATCH_LMOVE:
-				do_lmovebatch(&master);
-				break;
+				  do_post_filter(&master);
+				  do_lmovebatch(&master);
+				  break;
 			  case BATCH_LIHAVE:
-				do_localpost(&master);
-				break;  
+				  do_post_filter(&master);
+				  do_localpost(&master);
+				  break;  
 			  default:
-				break;
+				  break;
 			}
 		}
 		if((retval == RETVAL_NOARTICLES || retval == RETVAL_OK) && master.cleanup == TRUE) {
 			print_phrases(master.msgs, suck_phrases[7], NULL);
-			do_cleanup();
+			do_cleanup(&master);
 		}
+		
 		/* close out status log */
 		if(master.msgs != NULL && master.msgs != stdout && master.msgs != stderr) {
 			fclose(master.msgs);
@@ -517,9 +551,6 @@ int main(int argc, char *argv[]) {
 		/* free up group list */
 		while (master.groups != NULL) {
 			ptemp=(master.groups)->next;
-			if((master.groups)->group != NULL) {
-				free((master.groups)->group);
-			}
 			free(master.groups);
 			master.groups = ptemp;
 		}
@@ -609,8 +640,7 @@ int do_connect(PMaster master, int which_time) {
 		number(inbuf, &resp);
 		if(resp == 480 ) {
 			retval = do_authenticate(master);
-		}
-		
+		}		
 		if(retval == RETVAL_OK && master->do_modereader == TRUE) {
 			retval = send_command(master, "mode reader\r\n", &inbuf, 0);
 			if(retval == RETVAL_OK) {			
@@ -684,7 +714,6 @@ int get_message_index(PMaster master) {
 
 	if(retval == RETVAL_OK) {
 		print_phrases(master->msgs, suck_phrases[16], str_int(master->nritems), NULL);
-		retval = do_supplemental(master);
 	}
 	else if(ifp != NULL) {
 		/* this is in case we had to abort the above while loop (due to loss of pipe to server) and */
@@ -812,21 +841,20 @@ int do_one_group(PMaster master, char *buf, char *group, FILE *newrc, long lastr
 int get_articles(PMaster master) {
 
 	int retval, logcount, i;
-	FILE *logfp;
+	long loop, downloaded;
+	
 #ifdef HAVE_GETTIMEOFDAY
 	double bps;
 #endif
 	
 #ifdef KILLFILE
-	int ( *get_message)(PMaster, int); /* which function will we use get_one_article or get_one_article_kill) */
+	int ( *get_message)(PMaster, int, long); /* which function will we use get_one_article or get_one_article_kill) */
 
 	/* do we use killfiles? */
 	get_message = (master->killp == NULL) ? get_one_article : get_one_article_kill;
 #endif
 	retval = RETVAL_OK;
-	logfp = NULL;
-
-
+	downloaded = loop = 0; /* used to track how many downloaded, for reconnect option */
 	
 	/* figure out how many digits wide the articleCount is for display purposes */
 	/* this used to be log10()+1, but that meant I needed the math library */ 
@@ -838,19 +866,9 @@ int get_articles(PMaster master) {
 		retval = RETVAL_ERROR;
 	}
 	else {
-	 	if((logfp = fopen(full_path(FP_GET, FP_TMPDIR, N_RESTART), "w" )) == NULL) {
-			MyPerror(full_path(FP_GET, FP_TMPDIR, N_RESTART));
-			retval = RETVAL_ERROR;
-		}
- 		else {
-			/* do this here so we write out the restart for all articles */
-			/* already downloaded 	*/
+		retval = db_open(master);
+		if(retval == RETVAL_OK) {
 			master->curr = master->head;
-			for(i=0;i<master->itemon;i++) {
-				fprintf(logfp,"1");
-				master->curr = (master->curr)->next;
-			}
-			fflush(logfp);
 
 			if(master->batch == BATCH_INNFEED) {
 				/* open up batch file for appending to */
@@ -875,52 +893,58 @@ int get_articles(PMaster master) {
 				if(master->debug == TRUE) {
 					do_debug("Article nr = %s mandatory = %c\n", (master->curr)->msgnr, (master->curr)->mandatory);
 				}
-				/* write out restart */
-				fputc('1',logfp);
-  				fflush(logfp);
-				master->itemon++;
-				/* to be polite to the server, lets allow for a pause every once in a while */
-				if(master->pause_time > 0 && master->pause_nrmsgs > 0) {
-					if(master->itemon % master->pause_nrmsgs == 0) {
-						sleep(master->pause_time);
+				loop++;
+				if((master->curr)->downloaded == FALSE) {
+					/* we haven't yet downloaded this article */
+					downloaded++;
+					
+				        /* to be polite to the server, lets allow for a pause every once in a while */
+					if(master->pause_time > 0 && master->pause_nrmsgs > 0) {
+						if(downloaded % master->pause_nrmsgs == 0) {
+							sleep(master->pause_time);
+						}
 					}
-				}
-				if(master->status_file == FALSE && master->quiet == FALSE) {
-					/* if we are going to a file, we don't want all of these articles printed */
-					/* or if quiet flag is set */
-					/* this stuff doesn't go thru print_phrases so I can keep spacing right */
-					/* and I only print numbers and the BPS */
+					if(master->status_file == FALSE && master->quiet == FALSE) {
+						/* if we are going to a file, we don't want all of these articles printed */
+						/* or if quiet flag is set */
+						/* this stuff doesn't go thru print_phrases so I can keep spacing right */
+						/* and I only print numbers and the BPS */
 #ifdef HAVE_GETTIMEOFDAY
-					bps = TimerFunc(TIMER_GET_BPS, 0, master->msgs);
+						bps = TimerFunc(TIMER_GET_BPS, 0, master->msgs);
 #endif
-					if(master->use_gui == TRUE) {
-						/* this stuff is formatted by the GUI so we put it out in a format it likes */
+						if(master->use_gui == TRUE) {
+							/* this stuff is formatted by the GUI so we put it out in a format it likes */
 #ifndef HAVE_GETTIMEOFDAY
-						fprintf(master->msgs, "---%d+++\n",master->nritems - master->itemon);
+							fprintf(master->msgs, "---%ld+++\n",master->nritems - loop);
 #else
-						fprintf(master->msgs, "---%d+++%f\n", master->nritems - master->itemon, bps);
+							fprintf(master->msgs, "---%ld+++%f\n", master->nritems - loop, bps);
 #endif /* HAVE_GETTIMEOFDAY */
-					}
-					else {
+						}
+						else {
 #ifndef HAVE_GETTIMEOFDAY
-						fprintf(master->msgs, "%5d\r",master->nritems  - master->itemon);
+							fprintf(master->msgs, "%5ld\r",master->nritems  - loop);
 #else
-						fprintf(master->msgs, "%5d %9.1f %s\r",master->nritems-master->itemon,bps,suck_phrases[5]);
+							fprintf(master->msgs, "%5ld %9.1f %s\r",master->nritems-loop,bps,suck_phrases[5]);
 #endif /* HAVE_GETTIMEOFDAY */
+						}
+						fflush(master->msgs);	/* so message gets printed now */
 					}
-					fflush(master->msgs);	/* so message gets printed now */
-				}
 #ifdef KILLFILE
-				/* get one article */
-				retval = (*get_message)(master, logcount);
+				        /* get one article */
+					retval = (*get_message)(master, logcount, loop);
 #else
-				retval = get_one_article(master, logcount);
+					retval = get_one_article(master, logcount, loop);
 #endif
+					if(retval == RETVAL_OK ) {
+						retval = db_mark_dled(master, master->curr);
+					}
+				}
+				
 				master->curr = (master->curr)->next; 	/* get next article */
 
 				/* to be NOT polite to the server reconnect every X msgs to combat the INND */
 				/* LIKE_PULLERS=DONT.  This is last in loop, so if problem can abort gracefully */
-				if(master->reconnect_nr > 0 && master->itemon % master->reconnect_nr == 0) {
+				if(master->reconnect_nr > 0 && downloaded % master->reconnect_nr == 0) {
 					retval =  do_connect(master, CONNECT_AGAIN);
 					if(master->curr != NULL) {
 						(master->curr)->sentcmd = FALSE;	/* if we sent command force resend */
@@ -929,9 +953,9 @@ int get_articles(PMaster master) {
 
  
 		 	} /* end while */
-			fclose(logfp);
-			if(retval == RETVAL_OK && master->nritems == master->itemon) {
-				unlink(full_path(FP_GET, FP_TMPDIR, N_RESTART));
+			db_close(master);
+			if(retval == RETVAL_OK && master->nritems == loop) {
+				db_delete(master);
 			}
 
 			if(retval == RETVAL_OK) {
@@ -944,16 +968,14 @@ int get_articles(PMaster master) {
 	}
 	return retval;
 }
-/*----------------------------------------------------------------------*/
-/* add items from supplemental list to link list then write it out	*/
-/*----------------------------------------------------------------------*/ 
+/*-----------------------------------------------*/
+/* add items from supplemental list to link list */
+/* ----------------------------------------------*/ 
 int do_supplemental(PMaster master) {
 
-	int groupnr, retval, oldkept;
+	int retval, oldkept;
 	FILE *fp;
-	PList curr; 
 	char linein[MAXLINLEN+1];
-	PGroups grps;
 
 	retval = RETVAL_OK;
 	oldkept = master->nritems;		
@@ -973,51 +995,7 @@ int do_supplemental(PMaster master) {
 		fclose(fp);
 	}
 
-	if(master->head != NULL && master->nritems > 0) {
-		/* if we don't have any messages, we don't need to do this */
-		if(master->no_dedupe == FALSE) {
-			dedupe_list(master);
-		}
-#ifdef CHECK_HISTORY
-		if(master->do_chkhistory == TRUE) {
-			chkhistory(master);
-		}
-#endif
-		print_phrases(master->msgs,suck_phrases[20], str_int(master->nritems), NULL); 
 
-		/* create suck.restart */
-		if(master->head != NULL && master->nritems > 0) {
-			/* now write out whatever list we have*/
-			if((fp = fopen(full_path(FP_GET, FP_TMPDIR, N_SORTED), "w")) == NULL) {
-				retval = RETVAL_ERROR;
-				MyPerror(full_path(FP_GET, FP_TMPDIR, N_SORTED));
-			}
-			else {
-				curr = master->head;
-				groupnr = 0;
-				while(curr != NULL && retval == RETVAL_OK) {
-					if(curr->groupnr != groupnr) {
-						/* print out group name */
-						groupnr = curr->groupnr;
-						if(groupnr > 0) {
-							grps=master->groups;
-							while(grps->nr != groupnr) {
-								grps=grps->next;
-							}
-							fprintf(fp, "%s %s\n", RESTART_GROUP, grps->group);
-						}
-					}
-				
-					if(fprintf(fp, "%ld %s %c\n", curr->nr, curr->msgnr, curr->mandatory) == 0) {
-						retval = RETVAL_ERROR;
-						MyPerror(full_path(FP_GET, FP_TMPDIR, N_SORTED));
-					}
-					curr = curr->next;
-				}
-				fclose(fp);
-			}
-		}
-	}
 	return retval;
 }
 /*----------------------------------------------------------------------*/
@@ -1046,31 +1024,24 @@ int do_supplemental(PMaster master) {
 		 }
 		 else {
 			 grps->next = NULL;
-			 if((grps->group = malloc(strlen(grpname)+1)) == NULL) {
-				/* out of memory */
-				 error_log(ERRLOG_REPORT, suck_phrases[22], NULL);
-				 free(grps);
+			 strcpy(grps->group, grpname);
+				/* now add to list and count groupnr */
+			 if(master->groups == NULL) {
+				 grps->nr = 1;
+				 master->groups = grps;
 			 }
 			 else {
-				 strcpy(grps->group, grpname);
-				/* now add to list and count groupnr */
-				 if(master->groups == NULL) {
-					 grps->nr = 1;
-					 master->groups = grps;
+				 gptr = master->groups;
+				 while(gptr->next != NULL) {
+					 gptr = gptr->next;
 				 }
-				 else {
-					 gptr = master->groups;
-					 while(gptr->next != NULL) {
-						 gptr = gptr->next;
-					 }
-					 gptr->next = grps;
-					 grps->nr = gptr->nr + 1;
-				 }
-				 groupnr = grps->nr;
+				 gptr->next = grps;
+				 grps->nr = gptr->nr + 1;
+			 }
+			 groupnr = grps->nr;
 				 
-				 if(master->debug == TRUE) {
-					 do_debug("Adding to group list: %d %s\n", grps->nr, grps->group);
-				 }
+			 if(master->debug == TRUE) {
+				 do_debug("Adding to group list: %d %s\n", grps->nr, grps->group);
 			 }
 		 }
 	 }
@@ -1134,8 +1105,8 @@ int allocnode(PMaster master, char *linein, int mandatory, char *group, long msg
 				error_log(ERRLOG_REPORT, suck_phrases[22], NULL);
 				retval = RETVAL_ERROR;
 			}
-			else if((ptr->msgnr = calloc(strlen(st_ptr)+1, sizeof(char))) == NULL) {
-				error_log(ERRLOG_REPORT, suck_phrases[22], NULL);
+			else if(strlen(st_ptr) >= MAX_MSGID_LEN) {
+				error_log(ERRLOG_REPORT, suck_phrases[63], NULL);
 				free(ptr);
 				retval = RETVAL_ERROR;
 			}
@@ -1146,14 +1117,31 @@ int allocnode(PMaster master, char *linein, int mandatory, char *group, long msg
 				ptr->sentcmd = FALSE;		/* have we sent command to remote */
 				ptr->nr = msgnr;
 				ptr->groupnr = groupnr;
+				ptr->downloaded = FALSE;
+				ptr->dbnr = 0L;
+				ptr->delete = FALSE;
+				
 				if(master->debug == TRUE) {
 					do_debug("MSGID %s NR %d GRP %d MANDATORY %c added\n",ptr->msgnr, ptr->nr, ptr->groupnr, ptr->mandatory);
 				}
 			
 				/* now put on list */
 				if( curr == NULL) {
-				/* first node */
-					master->head = curr = ptr;
+					if(master->head == NULL) {
+				                /* first node */
+						master->head = curr = ptr;
+					}
+					else {
+						/* came in with a restart, find end of list */
+						curr = master->head;
+						while(curr->next != NULL) {
+							curr = curr->next;
+						}
+						
+						/* now add node on */
+						curr->next = ptr;
+						curr = ptr;
+					}	
 				}
 				else {
 					curr->next = ptr;
@@ -1167,9 +1155,8 @@ int allocnode(PMaster master, char *linein, int mandatory, char *group, long msg
 	return retval;
 }
 /*------------------------------------------------------------------------*/
-void free_one_node(PList node) {
-
-	free(node->msgnr);
+ void free_one_node(PList node) {
+		 
 	free(node);
 }
 /*----------------------------------------------------------------------------------*/
@@ -1221,7 +1208,7 @@ void free_one_node(PList node) {
 	 return cmd;
  }
 /*----------------------------------------------------------------------------------*/
-int get_one_article(PMaster master, int logcount) {
+int get_one_article(PMaster master, int logcount, long itemon) {
 
 	int nr, len, retval = RETVAL_OK;
 	char buf[MAXLINLEN+1];
@@ -1265,7 +1252,7 @@ int get_one_article(PMaster master, int logcount) {
 	if(master->MultiFile == TRUE) {
 		/* open file */
 		/* file name will be ####-#### ex 001-166 (nron,total) */
-		sprintf(buf,"%0*d-%d", logcount, master->itemon ,master->nritems);
+		sprintf(buf,"%0*ld-%d", logcount, itemon ,master->nritems);
 		/* the strcpy to avoid wiping out fname in second call to full_path */
 		strcpy(fname, full_path(FP_GET, FP_MSGDIR, buf));
 		strcat(buf, N_TMP_EXTENSION);		/* add tmp extension */
@@ -1400,61 +1387,36 @@ int get_a_chunk(int sockfd, FILE *fptr) {
 }
 /*-----------------------------------------------------------------------------*/
 int restart_yn(PMaster master) {
-	/* the restart file consists of 1 byte for each article prior to its */
-	/* being written.  So if its 2 bytes long, we've actually only got */
-	/* the first article.  If its zero bytes long, we aborted before we */
-	/* even got a chance to write the first byte */
-
+	int done, retval = RESTART_NO;
+	PList itemon;
+	long itemnr;
+	const char *fname;
 	struct stat buf;
-	int i, len, x, grpnr, retval = RESTART_NO;
-	FILE *fp;
-	char msgid[MAXLINLEN], group[MAXLINLEN], optional;
-	const char *fso;
-
-
-	strcpy(msgid,full_path(FP_GET, FP_TMPDIR, N_RESTART));
-	if(stat(msgid, &buf) == 0) {
-		/* If suck.restart exists something went wrong */
-		retval = RESTART_YES;	
-		master->itemon = (buf.st_size == 0) ? 0 : buf.st_size - 1;
-		/* the above is done since we write the byte before hand */
-		/* now build the link list in master */
-
-		/* do we skip the article we timed out on when we aborted?*/
-		if(master->skip_on_restart == TRUE) {
-			master->itemon++;
-			print_phrases(master->msgs, suck_phrases[60], NULL);
-		}
-		
-		fso = full_path(FP_GET, FP_TMPDIR, N_SORTED);
-		i = RETVAL_OK;
-		if((fp = fopen(fso, "r")) != NULL) {
-			len = strlen(RESTART_GROUP);
-			grpnr = 0;
-			while(fgets(msgid, MAXLINLEN, fp) !=  NULL) {
-				x = strlen(msgid);
-				/* strip off nl */
-				if(msgid[x-1] == '\n') {
-					msgid[--x] = '\0';
-				}
-				if(strncmp(msgid, RESTART_GROUP, len) == 0) {
-					/* got a group */
-					strcpy(group, &msgid[len+1]);
-				}
-				else {
-					optional = msgid[x];
-					msgid[x-1] = '\0';
-					/* restart recovery file is in format that allocnode needs */
-					i = allocnode(master, msgid, optional, group, 0L);
+	
+	fname = full_path(FP_GET, FP_TMPDIR, N_DBFILE);
+	if(stat(fname, &buf) == 0) {
+		/* restart file exists */
+		retval = db_read(master);
+		if(retval == RETVAL_OK) {
+			/* find out where we stopped */
+			itemon = master->head;
+			itemnr = 0L;
+			done = FALSE;
+			while(itemon != NULL && done == FALSE) {
+				itemnr++;
+				if(itemon->downloaded == FALSE) {
+					if(master->skip_on_restart == TRUE) {
+						/* mark one more */
+						print_phrases(master->msgs, suck_phrases[60], NULL);
+						itemon->downloaded = TRUE;
+					}
+					else {
+						itemnr--; /* don't count this one */
+					}
+					done = TRUE;
 				}
+				itemon = itemon->next;
 			}
-			fclose(fp);
-  			print_phrases(master->msgs, suck_phrases[2], str_int(master->itemon), str_int(master->nritems), NULL);
-		}
-		else {
-			error_log(ERRLOG_REPORT, suck_phrases[23],msgid,fso, NULL);
-			unlink(msgid);	/* so we have a clean start */
-			retval = RESTART_NO;
 		}
 	}
 	return retval;
@@ -1563,37 +1525,54 @@ void pause_signal(int action, PMaster master) {
 /*------------------------------------------------------------------------------*/
 /* 1. move N_OLDRC to N_OLD_OLDRC	N_OLDRC might not exist	       	        */
 /* 2. move N_NEWRC to N_OLDRC							*/
-/* 3. rm N_SORTED								*/
-/* 4. rm N_SUPPLEMENTAL								*/
+/* 3. rm N_SUPPLEMENTAL								*/
 /*------------------------------------------------------------------------------*/
-void do_cleanup() {
+void do_cleanup(PMaster master) {
 	const char *oldptr;
 	char ptr[PATH_MAX+1];
 	struct stat buf;
 	int exist;
-	
-	strcpy(ptr,full_path(FP_GET, FP_DATADIR, N_OLDRC));	
-	/* must strcpy since full path overwrites itself everytime */
-	oldptr = full_path(FP_GET, FP_DATADIR, N_OLD_OLDRC);
+	int okay = TRUE;
 
-	/* does the sucknewsrc file exist ? */
-	exist = TRUE;
-	if(stat(ptr, &buf) != 0 && errno == ENOENT) {
-		exist = FALSE;
+	if(master->debug == TRUE) {
+		do_debug("checking for existance of suck.newrc\n");
 	}
 	
-	if(exist == TRUE && move_file(ptr, oldptr) != 0) {
-		MyPerror(suck_phrases[30]);
+	if(stat(full_path(FP_GET, FP_TMPDIR, N_NEWRC), &buf) == 0) {
+		if(master->debug == TRUE) {
+			do_debug("found suck.newrc\n");
+		}
+		
+		/* we do the above test, in case we were in a restart */
+		/* with -R which would cause no suck.newrc to be created */
+		/* since message_index() would be skipped */
+		/* since no suck.newrc, don't move any of the sucknewrcs around */
+		strcpy(ptr,full_path(FP_GET, FP_DATADIR, N_OLDRC));	
+		/* must strcpy since full path overwrites itself everytime */
+		oldptr = full_path(FP_GET, FP_DATADIR, N_OLD_OLDRC);
+
+		/* does the sucknewsrc file exist ? */
+		exist = TRUE;
+		if(stat(ptr, &buf) != 0 && errno == ENOENT) {
+			exist = FALSE;
+		}
+		if(master->debug == TRUE) {
+			do_debug("sucknewsrc.old = %s sucknewsrc = %s exist = %s\n", oldptr, ptr, true_str(exist));
+		}
+		if(exist == TRUE && move_file(ptr, oldptr) != 0) {
+			MyPerror(suck_phrases[30]);
+			okay = FALSE;
+		}
+		else if(move_file(full_path(FP_GET, FP_TMPDIR, N_NEWRC), ptr) != 0) {
+			MyPerror(suck_phrases[31]);
+			okay = FALSE;
+		}
 	}
-	else if(move_file(full_path(FP_GET, FP_TMPDIR, N_NEWRC), ptr) != 0) {
-		MyPerror(suck_phrases[31]);
-	}
-	else if(unlink(full_path(FP_GET, FP_TMPDIR, N_SORTED)) != 0 && errno != ENOENT) {
-		/* ENOENT is not an error since this file may not always exist */
-		/* and it won't if there weren't any articles */
-		MyPerror(suck_phrases[32]);
-	}
-	else if(unlink(full_path(FP_GET, FP_DATADIR, N_SUPPLEMENTAL)) != 0 && errno != ENOENT) {
+	else if( errno != ENOENT) {
+		MyPerror(full_path(FP_GET, FP_DATADIR, N_NEWRC));
+		okay = FALSE;
+	}		
+	if(okay == TRUE && unlink(full_path(FP_GET, FP_DATADIR, N_SUPPLEMENTAL)) != 0 && errno != ENOENT) {
 		/* ENOENT is not an error since this file may not always exist */
 		MyPerror(suck_phrases[33]);
 	}
@@ -1830,6 +1809,15 @@ int parse_args(PMaster master, int arg, char *argv[]) {
 	case ARG_CONN_DEDUPE:
 		master->conn_dedupe = TRUE;
 		break;
+	case ARG_POST_FILTER:
+		master->post_filter = argv[0];
+		break;
+	case ARG_CONN_ACTIVE:
+		master->conn_active = TRUE;
+		break;
+	case ARG_HIST_FILE:
+		master->history_file = argv[0];
+		break;
 #ifdef TIMEOUT
 	case ARG_TIMEOUT:
 		TimeOut = atoi(argv[0]);
