@@ -2,20 +2,24 @@
 #define _SUCK_SUCK_H 1
 
 #include <stdio.h>	/* for FILE */
+#include "suck_config.h"
 
 /* link list structure one for each article */
 typedef struct LinkList {
 	struct LinkList *next;
-	char *msgnr;
-	char mandatory;
-	int  sentcmd;
+	char msgnr[MAX_MSGID_LEN];
 	int groupnr;
 	long nr;
+	long dbnr;
+	char mandatory;
+	char downloaded;
+	char delete;
+	char sentcmd;
 } List, *PList;
 
 /* link list for group names */
 typedef struct GroupList {
-	char *group;
+	char group[MAX_GRP_LEN];
 	int nr;
 	struct GroupList *next;
 } Groups, *PGroups;
@@ -25,7 +29,6 @@ typedef struct {
 	PList head;
 	PList curr;
 	int nritems;
-	int itemon;
 	int nrgot;
 	int sockfd;
 	int MultiFile;
@@ -56,10 +59,12 @@ typedef struct {
 	int use_gui;
 	int do_xover;
 	int conn_dedupe;
+	int conn_active;
 	unsigned short int portnr;
 	long rnews_size;
 	FILE *msgs;
 	FILE *innfeed;
+	int db;
 	const char *userid;
 	const char *passwd;
 	const char *host;
@@ -70,6 +75,8 @@ typedef struct {
 	const char *localhost;
 	const char *activefile;
 	const char *kill_log_name;
+	const char *post_filter;
+	const char *history_file;
 	PGroups groups;
 	int grpnr;
 	void *killp;
@@ -81,14 +88,16 @@ void free_one_node(PList);
 int send_command(PMaster, const char *, char **, int);
 int get_message_index(PMaster);
 int do_one_group(PMaster, char *, char *, FILE *, long, int);
-int do_supplemental(PMaster);
 const char *build_command(PMaster, const char *, PList);
 const char *null_str(const char *);
 const char *true_str(int);
 int allocnode(PMaster, char *, int, char *, long);
+int do_connect(PMaster, int);
 
 enum { RETVAL_ERROR = -1, RETVAL_OK = 0, RETVAL_NOARTICLES, RETVAL_UNEXPECTEDANS, RETVAL_VERNR, RETVAL_NOAUTH, RETVAL_EMPTYKILL };
 enum { BATCH_FALSE, BATCH_INNXMIT, BATCH_RNEWS, BATCH_LMOVE, BATCH_INNFEED, BATCH_LIHAVE };		/* poss values for batch variable */
-enum { MANDATORY_YES = 'M' , MANDATORY_OPTIONAL = 'O' };	/* do we have to download this article */
+enum { MANDATORY_YES = 'M' , MANDATORY_OPTIONAL = 'O'};	/* do we have to download this article */
+/* note the MANDATORY_DELETE is used in the dedupe and checkhistory phases to flag em for deletion */
+enum {CONNECT_FIRST, CONNECT_AGAIN};
 
 #endif /* _SUCK_SUCK_H */
